/*
 * Copyright 1998 Vectaport Inc.
 *
 * Permission to use, copy, modify, distribute, and sell this software and its
 * documentation for any purpose is hereby granted without fee, provided
 * that the above copyright notice appear in all copies and that both that
 * copyright notice and this permission notice appear in supporting
 * documentation, and that the names of the copyright holders not be used in 
 * advertising or publicity pertaining to distribution of the software without 
 * specific, written prior permission.  The copyright holders make no 
 * representations about the suitability of this software for any purpose.  
 * It is provided "as is" without express or implied warranty.
 *
 * THE COPYRIGHT HOLDERS DISCLAIM ALL WARRANTIES WITH REGARD TO THIS SOFTWARE,
 * INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS.
 * IN NO EVENT SHALL THE COPYRIGHT HOLDERS BE LIABLE FOR ANY SPECIAL,
 * INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING
 * FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT,
 * NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION
 * WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 */

#include <VhclUnidraw/vhclabout.h>
#include <VhclUnidraw/vhcleditor.h>
#include <VhclUnidraw/vhclkit.h>
#include <UtmUnidraw/utmcmds.h>
#include <MapUnidraw/mapcomps.h>
#include <MapUnidraw/mapexport.h>
#include <MapUnidraw/mapimport.h>
#include <FrameUnidraw/framecmds.h>
#include <FrameUnidraw/frameeditor.h>
#include <FrameUnidraw/framekit.h>
#include <FrameUnidraw/framestates.h>
#include <OverlayUnidraw/ovcmds.h>
#include <OverlayUnidraw/ovcmds.h>
#include <OverlayUnidraw/ovprint.h>
#include <UniIdraw/idkybd.h>
#include <Unidraw/kybd.h>
#include <Unidraw/ctrlinfo.h>
#include <IVGlyph/exportchooser.h>
#include <IVGlyph/importchooser.h>
#include <IV-look/kit.h>
#include <InterViews/layout.h>
#include <InterViews/session.h>
#include <InterViews/style.h>


VhclKit* VhclKit::_vhclkit = nil;

VhclKit::VhclKit()
: UtmKit()
{
}

VhclKit::~VhclKit() {
}

VhclKit* VhclKit::Instance() {
    if (!_vhclkit)
	_vhclkit = new VhclKit();
    return _vhclkit;
}

MenuItem * VhclKit::MakeFileMenu() {
    LayoutKit& lk = *LayoutKit::instance();
    WidgetKit& kit = *WidgetKit::instance();
    
    MenuItem *mbi = kit.menubar_item(kit.label("File"));
    mbi->menu(kit.pulldown());
    MakeMenu(mbi, new VhclAboutCmd(new ControlInfo("About vhclviewer", KLBL_ABOUT, CODE_ABOUT)),
	     "About vhclviewer");
    MakeMenu(mbi, new OvNewCompCmd(new ControlInfo("New", KLBL_NEWCOMP, CODE_NEWCOMP),
				 new MapIdrawComp),
	     "New   ");
    MakeMenu(mbi, new OvRevertCmd(new ControlInfo("Revert", KLBL_REVERT, CODE_REVERT)),
	     "Revert   ");
    MakeMenu(mbi, new UtmOpenCmd(new ControlInfo("Open Map...", KLBL_VIEWCOMP, CODE_VIEWCOMP)),
	     "Open Map...   ");
    MakeMenu(mbi, new OvOpenCmd(new ControlInfo("Open Graphic...", "", "")),
	     "Open Graphic...   ");
    MakeMenu(mbi, new OvSaveCompCmd(new ControlInfo("Save", KLBL_SAVECOMP, CODE_SAVECOMP)),
	     "Save   ");
    MakeMenu(mbi, new OvSaveCompAsCmd(new ControlInfo("Save As...",
					    KLBL_SAVECOMPAS,
					    CODE_SAVECOMPAS)),
	     "Save As...   ");
    MakeMenu(mbi, new OvPrintCmd(new ControlInfo("Print...", KLBL_PRINT, CODE_PRINT)),
	     "Print...   ");
    Style * istyle = new Style(Session::instance()->style());
    istyle->attribute("subcaption", "Import map features from file:");
    istyle->attribute("open", "Import");
    ImportChooser* ichooser = new ImportChooser(".", WidgetKit::instance(), istyle, nil, false, true);
    Resource::ref(ichooser);
    MakeMenu(mbi, new MapImportCmd(new ControlInfo("Import Map Features...",
						  KLBL_IMPORT,
						  CODE_IMPORT),
				   ichooser),
	     "Import Map Features...   ");
    Style * estyle = new Style(Session::instance()->style());
    estyle->attribute("subcaption", "Export selected map features to file:");
    estyle->attribute("open", "Export");
    const char *formats[] = {"idraw", "drawtool"};
    ExportChooser* echooser = new ExportChooser(".", WidgetKit::instance(), estyle,
				 formats, sizeof(formats)/sizeof(char*), nil, nil, true, false);
    Resource::ref(echooser);
    MakeMenu(mbi, new MapExportCmd(new ControlInfo("Export Map Features...",
						  "^X", "\030"),
			       echooser),
	     "Export Map Features...   ");
    MakeMenu(mbi, new OvWindowDumpAsCmd(new ControlInfo("Dump Window As..."
						  )),
	     "Dump Window As...   ");
    mbi->menu()->append_item(kit.menu_item_separator());
    MakeMenu(mbi, new OvQuitCmd(new ControlInfo("Quit", KLBL_QUIT, CODE_QUIT)),
	     "Quit   ");

    return mbi;
}

MenuItem* VhclKit::MakeFrameMenu() {
    LayoutKit& lk = *LayoutKit::instance();
    WidgetKit& kit = *WidgetKit::instance();
    
    MenuItem *mbi = kit.menubar_item(kit.label("Frame"));
    mbi->menu(kit.pulldown());

    MoveFrameCmd::FuncEnable("simtime(%d)", "simtime(%d :abs)");
    MakeMenu(mbi, new MoveFrameCmd(new ControlInfo("Move Forward","^F",""), +1),
	     "Move Forward   ");
    MakeMenu(mbi, new MoveFrameCmd(new ControlInfo("Move Backward","^B",""), -1),
	     "Move Backward   ");

    MakeMenu(mbi, new FrameBeginCmd(new ControlInfo("Goto First Frame")),
	     "Goto First Frame");
    MakeMenu(mbi, new FrameEndCmd(new ControlInfo("Goto Last Frame")),
	     "Goto Last Frame ");
    mbi->menu()->append_item(kit.menu_item_separator());

    MakeMenu(mbi, new CreateMoveFrameCmd(new ControlInfo("New Forward","F","F"), true),
	     "New Forward    ");
    MakeMenu(mbi, new CreateMoveFrameCmd(new ControlInfo("New Backward","B","B"), false),
	     "New Backward   ");
    MakeMenu(mbi, new CopyMoveFrameCmd(new ControlInfo("Copy Forward","",""),
				       true),
	     "Copy Forward   ");
    MakeMenu(mbi, new CopyMoveFrameCmd(new ControlInfo("Copy Backward","",""), 
				       false),
	     "Copy Backward  ");
    MakeMenu(mbi, new DeleteFrameCmd(new ControlInfo("Delete","D","D")),
	     "Delete  ");
    mbi->menu()->append_item(kit.menu_item_separator());
    MakeMenu(mbi, new ShowOtherFrameCmd(new ControlInfo("Show Prev Frame","",""), -1),
	     "Show Prev Frame");
    MakeMenu(mbi, new ShowOtherFrameCmd(new ControlInfo("Hide Prev Frame","",""), 0),
	     "Hide Prev Frame");
    return mbi;
}

void VhclKit::InitLayout(const char* name) {
  FrameKit::InitLayout(this, name);
}

Glyph* VhclKit::MakeStates() {
    ((FrameEditor*)_ed)->framenumstate() = new FrameNumberState();
    ((FrameEditor*)_ed)->frameliststate() = new FrameListState();
    ((VhclEditor*)_ed)->simtimestate() = new FrameNumberState(0, "Simulation Time");
    NameView* fnumview = new NameView(((FrameEditor*)_ed)->framenumstate());
    NameView* flistview = new NameView(((FrameEditor*)_ed)->frameliststate());
    NameView* simtimeview = new NameView(((VhclEditor*)_ed)->simtimestate());

    LayoutKit& lk = *LayoutKit::instance();
    WidgetKit& kit = *WidgetKit::instance();
    return kit.inset_frame(
	lk.margin(
	    lk.hbox(fnumview, lk.hspace(40), flistview, lk.hglue()),
	    4, 2
	    )
    );
		
}
